#define iterations 30000
#define nbins 8

#include <stdio.h>
#include "fissionEvent.h"

void init(void);
FILE* openfile(char* name);
void output(int* hist);

int main() {
   int isotope = 92235;
   double energy_MeV = 1.;
   double nubar = 2.523670;
   double time = 0.;

   int maxerrorlength=10000;
   char errors[maxerrorlength];

   int hist[nbins];
   for (int i=0; i<nbins; i++) hist[i] = 0.;

   init();
   for (int i=0; i<iterations; i++) {
      fissionEvent* fe = new fissionEvent(isotope, time, nubar, energy_MeV, 1);
      int errorlength=maxerrorlength;
      fe->getFREYAerrors(&errorlength, &errors[0]);
      if (errorlength>1) {
         printf("%s\n",errors);
         exit(1);
      }
      int nneutrons = fe->getNeutronNu();
      hist[nneutrons]++;
      delete fe;
   }
   output(hist);
}

void init(void) {
   unsigned short int s[3] = {1234, 5678, 9012};
   int i;
   seed48(s);
   fissionEvent::setCorrelationOption(3);
   return;
}

FILE* openfile(char* name) {
   FILE* fp = fopen(name, "w");
   if (fp == (FILE *) 0) fprintf(stderr, "Could not open %s for writing", name);
   return fp;
}

void output(int* hist) {
   char filename [1024];
   sprintf(filename, "nu_dist.res");
   FILE* fp = openfile(filename);

   unsigned int sum=0;
   for (int i=0; i<nbins; i++) sum += hist[i];
   for (int i=0; i<nbins; i++) fprintf(fp, "%d : %f\n", i, 1.*hist[i]/sum);

   fclose(fp);
   return;
}

